@@ -61,8 +61,7 @@ extern	int		l7vs_lsock_select_service(
 				char*,
 				size_t,
 				struct l7vs_service**,
-				struct l7vs_dest**,
-				int* );
+				struct l7vs_dest**);
 /*!
  * serialize struct sockaddr_in
  *
